package org.richfaces.testng;

import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.richfaces.Templates;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class DropDownMenuTest extends SeleniumTestBase implements RichSeleniumTest {

    public DropDownMenuTest() {
        super("http", "localhost", serverPort);
    }

    /**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
        super.startSelenium(browser);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testDropDownMenuComponent() throws Exception {
        _testDropDownMenuComponent(Templates.SIMPLE);
        _testDropDownMenuComponent(Templates.DATATABLE);
        _testDropDownMenuComponent(Templates.MODALPANEL);
    }

    private void _testDropDownMenuComponent(Templates template) {
        renderPage(getTestUrl(), template);

        String parentId = getParentId() + "_form:";
        String file = parentId + "file";
        String open = parentId + "open:anchor";
        String saveAs = parentId + "saveAs:anchor";
        String save = parentId + "save:anchor";
        String saveAll = parentId + "saveAll:anchor";
        String close = parentId + "close:anchor";
        String exit = parentId + "exit:anchor";
        String operation = parentId + "operation";

        writeStatus("Check menu item in ajax mode");

        selenium.mouseOver(file);
        selenium.mouseOver(saveAs);
        clickById(saveAll);

        AssertTextEquals(operation, "Save All");

        selenium.mouseOver(file);
        selenium.mouseOver(saveAs);
        clickById(save);

        AssertTextEquals(operation, "Save");

        writeStatus("Check the drop down menu closed");

        AssertNotVisible(save);
        AssertNotVisible(saveAs);

        writeStatus("Check menu item in server mode");

        selenium.mouseOver(file);
        clickCommandAndWait(close);

        AssertTextEquals(operation, "Close");

        writeStatus("Check menu item in 'none' mode");

        selenium.mouseOver(file);
        selenium.mouseOver(exit);
        clickById(exit);

        AssertTextEquals(operation, "Close");
        AssertNotVisible(exit);

    }

	public String getTestUrl() {
		return "/faces/pages/dropDownMenu/dropDownMenuTest.xhtml";
	}
}
