package org.richfaces.testng;

import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.richfaces.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class InplaceSelectTest extends SeleniumTestBase implements RichSeleniumTest {

    public InplaceSelectTest() {
        super("http", "localhost", serverPort);
    }

    /**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
        super.startSelenium(browser);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testInplaceSelectComponent() throws Exception {
        _testInplaceSelectComponent(Templates.SIMPLE);
        _testInplaceSelectComponent(Templates.DATATABLE);
        _testInplaceSelectComponent(Templates.MODALPANEL);
    }

    private void _testInplaceSelectComponent(Templates template) {
        renderPage(getTestUrl(), template);

        String parentId = getParentId() + "_form:";
        String inplaceSelectId = parentId + "inplaceSelect";
        String okButton = parentId + "inplaceSelect" + "ok";
        //String inplaceTmpValueElemId = parentId + "inplaceSelect" + "inplaceTmpValue";
        String cancelButton = parentId + "inplaceSelect" + "cancel";
        String popupId = "list" + inplaceSelectId;

        writeStatus("click the component");

        clickById(inplaceSelectId);
        Assert.assertFalse(isVisibleById(popupId), "Component pop-up should not show up on solitary click");

        writeStatus("double click the component");

        selenium.doubleClick(inplaceSelectId);
        Assert.assertTrue(isVisibleById(popupId), "Component pop-up should show up on double click");

        writeStatus("Check that controls buttons are present");

        Assert.assertTrue(isPresentById(okButton));
        Assert.assertTrue(isPresentById(cancelButton));

        writeStatus("Select second element"); //Birch

        selenium.mouseMove("xpath=//div[@id='list" + inplaceSelectId + "']/span[2]");
        selenium.fireEvent(okButton, "mousedown");

        writeStatus("Check that a new element is selected");

        Assert.assertEquals(getValue(inplaceSelectId), "Birch");

        writeStatus("Select another element"); // Aspen

        selenium.doubleClick(inplaceSelectId);

        selenium.mouseMove("xpath=//div[@id='list" + inplaceSelectId + "']/span[3]");
        selenium.fireEvent(cancelButton, "mousedown");

        writeStatus("Cancel selected value.");
        Assert.assertEquals(getValue(inplaceSelectId), "Birch", "A value has not to be changed");

        writeStatus("Verify javascript event triggering");

        runScript("theLatestEvent='undefined'");

        writeStatus("Double click the component");
        selenium.doubleClick(inplaceSelectId);

        Assert.assertEquals(runScript("theLatestEvent"), "editactivated", "editactivated event has not been triggered");

        writeStatus("Stop editing with ok");
        selenium.fireEvent(okButton, "mousedown");
        Assert.assertEquals(runScript("theLatestEvent"), "viewactivated", "viewactivated event has not been triggered");

//      viewactivated event is not fired ... almost sure bug
//      writeStatus("Stop editing with cancel");
//      selenium.fireEvent(cancelButton, "mousedown");
//      Assert.assertEquals(runScript("theLatestEvent"), "viewactivated", "viewactivated event has not been triggered");

    }

    /**
     * Gets component value.
     * Note: JavaScript component API getValue() doesn't exit 
     * @param clientId
     * @return 
     */
    private String getValue(String clientId) {
        return runScript("$('" + clientId + "inplaceValue').value;");
    }

	public String getTestUrl() {
		return "/faces/pages/inplaceSelect/inplaceSelectTest.xhtml";
	}

}
