package org.richfaces.testng;

import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.richfaces.Templates;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

public class KeepAliveTest extends SeleniumTestBase implements RichSeleniumTest {

    public KeepAliveTest() {
        super("http", "localhost", serverPort);
    }

    /**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
        super.startSelenium(browser);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testKeepAliveComponent() throws Exception {
    	_testKeepAliveComponent(Templates.SIMPLE);
        _testKeepAliveComponent(Templates.DATATABLE);
        _testKeepAliveComponent(Templates.MODALPANEL);
    }

    private void _testKeepAliveComponent(Templates template) {
        renderPage(getTestUrl(), template);

        String parentId = getParentId();

        String sumElemId = parentId + "sum";
        AssertValueEquals(sumElemId, "0");
        String addButtonId = parentId + "form:add";
        String addAjaxButtonId = parentId + "form:addAjax";

        writeStatus("The sum has to be increased by 5 twice");

        clickCommandAndWait(addButtonId);
        clickCommandAndWait(addButtonId);
        AssertValueEquals(sumElemId, "10");

        writeStatus("The sum has to be increased by 5 twice again");

        clickById(addAjaxButtonId);
        clickById(addAjaxButtonId);
        waitForAjaxCompletion();
        AssertValueEquals(sumElemId, "20");

        writeStatus("Test ajax only regime ... ");

        String sumAjaxOnlyElemId = parentId + "sumAjaxOnly";
        AssertValueEquals(sumAjaxOnlyElemId, "0");
        addButtonId = parentId + "formAjaxOnly:add";
        addAjaxButtonId = parentId + "formAjaxOnly:addAjax";

        writeStatus("State is not saved between full submits. The sum always is 5");

        clickCommandAndWait(addButtonId);
        clickCommandAndWait(addButtonId);
        AssertValueNotEquals(sumAjaxOnlyElemId, "10");

        writeStatus("Ajax request! The sum has to be increased by 5 twice");

//      This test doesn't pass! Bug?!

//      clickById(addAjaxButtonId);
//      clickById(addAjaxButtonId);
//      waitForAjaxCompletion();
//      AssertValueEquals(sumAjaxOnlyElemId, "10");
    }

	public String getTestUrl() {
		return "/faces/pages/keepAlive/keepAliveTest.xhtml";
	}
}
