/**
 * License Agreement.
 *
 * Rich Faces - Natural Ajax for Java Server Faces (JSF)
 *
 * Copyright (C) 2007 Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.renderkit;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.faces.component.UIForm;
import javax.faces.component.UISelectItem;
import javax.faces.component.UISelectItems;
import javax.faces.component.html.HtmlForm;
import javax.faces.model.SelectItem;

import org.ajax4jsf.tests.AbstractAjax4JsfTestCase;
import org.richfaces.component.UIPickList;
import org.richfaces.renderkit.html.PickListRendererGen;

import com.gargoylesoftware.htmlunit.html.HtmlElement;
import com.gargoylesoftware.htmlunit.html.HtmlPage;

/**
 * @author Anton Belevich 
 *
 */
public class PickListRendererTest extends AbstractAjax4JsfTestCase {
	
	private UIForm form;
	private UIPickList pickList;
	private UIPickList pickList2;
	private String [] selected = new String [] {"District of Columbia,Illinois,Maryland,New Hampshire,New Jersey"};
	private List selectedList = new ArrayList();
	private static Set javaScripts = new HashSet();
	private PickListRendererGen  renderer;
	
	List selectItems = new ArrayList();


	static {
		javaScripts.add("a4j_3_2_0-SNAPSHOTorg.ajax4jsf.javascript.PrototypeScript");
		javaScripts.add("a4j_3_2_0-SNAPSHOTscripts/ShuttleUtils.js");
		javaScripts.add("a4j_3_2_0-SNAPSHOTscripts/SelectItem.js");
		javaScripts.add("a4j_3_2_0-SNAPSHOTscripts/PickListSI.js");
		javaScripts.add("a4j_3_2_0-SNAPSHOTscripts/LayoutManager.js");
		javaScripts.add("a4j_3_2_0-SNAPSHOTscripts/Control.js");
		javaScripts.add("a4j_3_2_0-SNAPSHOTscripts/ListBase.js");
		javaScripts.add("a4j_3_2_0-SNAPSHOTscripts/OrderingList.js");
		javaScripts.add("a4j_3_2_0-SNAPSHOTscripts/ListShuttle.js");
		javaScripts.add("a4j_3_2_0-SNAPSHOTscripts/PickList.js");
	}
    
	public PickListRendererTest(String name) {
		super(name);
		selectedList.add("District of Columbia");
		selectedList.add("Illinois");
		selectedList.add("Maryland");
		selectedList.add("New Hampshire");
		selectedList.add("New Jersey");
	}
	
	public void setUp() throws Exception {
		super.setUp();
		renderer = new PickListRendererGen();
		form = new HtmlForm();
		form.setId("form");
		facesContext.getViewRoot().getChildren().add(form);
		
		pickList = (UIPickList)application.createComponent("org.richfaces.PickList");
	    pickList.setValue(selected);
			
	    selectItems.add(new SelectItem("District of Columbia"));
		selectItems.add(new SelectItem("Illinois"));
		selectItems.add(new SelectItem("Maryland"));
		selectItems.add(new SelectItem("Nevada"));
		selectItems.add(new SelectItem("New Hampshire"));
		selectItems.add(new SelectItem("New Jersey"));
		    
		UISelectItem item1 = new UISelectItem();
		item1.setValue(new SelectItem("Oregon"));
			
		UISelectItem item2 = new UISelectItem();
		item2.setValue(new SelectItem("Pennsylvania"));
			
		UISelectItem item3 = new UISelectItem();
		item3.setValue(new SelectItem("Rhode Island"));
			
		UISelectItem item4 = new UISelectItem();
		item4.setValue(new SelectItem("South Carolina"));
			
	    pickList.getChildren().add(item1);
	    pickList.getChildren().add(item2);
	    pickList.getChildren().add(item3);
	    pickList.getChildren().add(item4);
		    
	    UISelectItems items = new UISelectItems();
	    items.setValue(selectItems);
	    form.getChildren().add(pickList);
	    
	    pickList2 = (UIPickList)application.createComponent("org.richfaces.PickList");
	    pickList2.setValue(selectedList);
	    pickList2.getChildren().add(item1);
	    pickList2.getChildren().add(item2);
	    pickList2.getChildren().add(item3);
	    pickList2.getChildren().add(item4);
	    form.getChildren().add(pickList2);
	}
	
	public void testRender(){
		
		try {
			HtmlPage page = renderView();
			assertNotNull(page);
			HtmlElement elem = page.getHtmlElementById(pickList.getClientId(facesContext));
			assertNotNull(elem);
			assertEquals(elem.getTagName(), "table");
			renderer.doEncodeEnd(writer, facesContext, pickList);
			
			HtmlElement elem2 = page.getHtmlElementById(pickList2.getClientId(facesContext));
			assertNotNull(elem2);
			assertEquals(elem.getTagName(), "table");
			renderer.doEncodeEnd(writer, facesContext, pickList);
						
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail();
		}
	}
	

}
